#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging
import os
from xml.sax import handler, make_parser

from django.core.management.base import BaseCommand
from django.db.models import Max
from django.db import connection

from common.models import ImportInProgress
from importer.WalentyPreprocessXML import WalentyPreprocessTeiHandler
from importer.WalentyXML import WalentyTeiHandler
from semantics.models import Argument, Frame
from shellvalier.settings import BASE_DIR


class Command(BaseCommand):
    args = 'none'
    help = ''

    def handle(self, **options):
        update_tei()


def update_tei():
    logging.basicConfig(filename='import.log', level=logging.DEBUG)

    xml_file_name = os.getenv('WALENTY_FILE_NAME', default='walenty_20210913_smaller.xml')

    print("Loading walenty dict from: {}".format(xml_file_name))

    xml_file = os.path.join(BASE_DIR, 'data', 'walenty', xml_file_name)

    xml_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), xml_file)

    parser = make_parser()
    parser.setFeature(handler.feature_external_ges, False)

    parser.setContentHandler(WalentyPreprocessTeiHandler())
    parser.parse(xml_path)

    entry_meanings = parser.getContentHandler().entry_meanings
    meanings = parser.getContentHandler().meanings
    frames = parser.getContentHandler().frames

    parser.setContentHandler(WalentyTeiHandler(entry_meanings, meanings, frames))
    parser.parse(xml_path)
    ImportInProgress.objects.all().delete()

    argument_max_id = Argument.objects.aggregate(Max('id'))
    cursor = connection.cursor()
    cursor.execute("Select setval('semantics_argument_id_seq', {} )".format(argument_max_id['id__max'] + 1))
    cursor.close()

    frame_max_id = Frame.objects.aggregate(Max('id'))
    cursor = connection.cursor()
    cursor.execute("Select setval('semantics_frame_id_seq', {} )".format(frame_max_id['id__max'] + 1))
    cursor.close()

    cursor = connection.cursor()
    cursor.execute("create index if not exists meanings_lexicalunit_base_index on meanings_lexicalunit (base);")
    cursor.close()

    cursor = connection.cursor()
    cursor.execute("create index if not exists connections_entry_name_index on connections_entry (name);")
    cursor.close()


